<?php

/**
 * @license    GPL 2 (https://www.gnu.org/licenses/gpl.html)
 *
 * @author Wizzard <wizzardsk@gmail.com>
 */
$lang['menu']                  = 'Prehľad potvrdení';
$lang['menu_assign']           = 'Priradenia potvrdení';
$lang['ackRequired']           = 'Musíte potvrdiť túto stránku';
$lang['ackText']               = 'Potvrdzujem, že som si prečítal a porozumel obsahu tejto stránky';
$lang['ackButton']             = 'Potvrdiť';
$lang['ackOk']                 = 'Stránka potvrdená';
$lang['ackGranted']            = 'Túto stránku ste potvrdili %s';
$lang['ackDiff']               = 'Zobraziť rozdiely od vášho posledného potvrdenia z %s';
$lang['ackNotFound']           = 'Nič sa nenašlo';
$lang['overviewPage']          = 'Stránka';
$lang['overviewUser']          = 'Používateľ';
$lang['overviewTime']          = 'Posledné potvrdenie';
$lang['overviewMod']           = 'Posledná zmena';
$lang['overviewCurrent']       = 'Aktuálne?';
$lang['overviewHistory']       = 'K predchádzajúcim potvrdeniam sa dostanete priamym dopytom do databázy. Požiadajte o pomoc administrátora.';
$lang['count']                 = 'Používateľ %s potvrdil %d z %d stránok.';
$lang['yes']                   = 'áno';
$lang['home']                  = 'Najnovšie potvrdenia';
$lang['status']                = 'Stav';
$lang['all']                   = 'všetky';
$lang['current']               = 'aktuálne';
$lang['due']                   = 'čakajúce';
$lang['toomanyresults']        = 'Príliš veľa výsledkov na zobrazenie. Výstup obmedzený.';
$lang['pattern']               = 'Vzor stránky';
$lang['assignees']             = 'Priradení';
$lang['save']                  = 'Uložiť';
$lang['error sqlite plugin missing'] = 'Chýba sqlite plugin';
