<?php
/**
 * German language file for acknowledge plugin
 *
 * @author Andreas Gohr, Anna Dabrowska <dokuwiki@cosmocode.de>
 */

// menu entry for admin plugins
$lang['menu'] = 'Bestätigungen: Überblick';
$lang['menu_assign'] = 'Bestätigungen: Zuweisungen';
$lang['menu_stats'] = 'Bestätigungen: Statistik';

// custom language strings for the plugin
$lang['ackRequired'] = 'Sie müssen diese Seite noch bestätigen';
$lang['ackText'] = 'Ich bestätige den Inhalt dieser Seite gelesen und verstanden zu haben';
$lang['ackButton'] = 'Bestätigen';
$lang['ackGranted'] = 'Sie haben diese Seite am %s bestätigt.';
$lang['ackDiff'] = 'Zeige Unterschiede zur letzten bestätigten Revision von %s';
$lang['ackNotFound'] = 'Nichts gefunden';
$lang['reportTitle'] = 'Bestätigungsbericht';
$lang['reportAcknowledgedTitle'] = 'Nutzer, die diese Seite bestätigt haben';
$lang['reportPendingTitle'] = 'Nutzer, die diese Seite noch bestätigen müssen';
$lang['reportNobody'] = '(noch keine)';
$lang['reportUserCount'] = '%d Nutzer (anzeigen)';
$lang['reportAckedOn'] = 'am';
$lang['overviewPage'] = 'Seite';
$lang['overviewUser'] = 'Nutzer';
$lang['overviewTime'] = 'Letzte Bestätigung';
$lang['overviewMod'] = 'Letzte Änderung';
$lang['overviewCurrent'] = 'Aktuell?';
$lang['overviewHistory'] = 'Sie können ältere Bestätigungen über die Datenbank abfragen. Bitten Sie Ihren Administrator um Hilfe.';
$lang['count'] = 'Nutzer %s hat %d von %d Seiten bestätigt.';
$lang['yes'] = 'ja';
$lang['home'] = 'Neueste Bestätigungen';
$lang['status'] = 'Status';
$lang['all'] = 'alle';
$lang['current'] = 'aktuell';
$lang['due'] = 'offen';

$lang['toomanyresults'] = 'Zu viele Ergebnisse. Die Ausgabe wurde begrenzt.';

$lang['statsTotal'] = 'Gesamtes Wiki';
$lang['statsNamespace'] = 'Namensraum';
$lang['statsSubnamespace'] = 'Sub-Namensräume';
$lang['statsRoot'] = '(oberste Ebene)';
$lang['statsHere'] = '(Seiten in diesem Namensraum)';
$lang['statsAcked'] = 'Bestätigt';
$lang['statsRequired'] = 'Erforderlich';
$lang['statsRatio'] = 'Fortschritt';
$lang['statsPages'] = 'Zugewiesene Seiten';

$lang['pattern'] = 'Seiten-Muster';
$lang['assignees'] = 'Zuweisung';
$lang['save'] = 'Speichern';

$lang['error sqlite plugin missing'] = 'Das sqlite plugin ist nicht installiert';

$lang['notification'] = 'Sie müssen bestätigen, dass Sie die folgende Seite gelesen haben: %s';
