<?php
/**
 * English language file for acknowledge plugin
 *
 * @author Andreas Gohr, Anna Dabrowska <dokuwiki@cosmocode.de>
 */

// menu entry for admin plugins
$lang['menu'] = 'Latest Acknowledgements';
$lang['menu_assign'] = 'Acknowledgement Assignments';
$lang['menu_stats'] = 'Acknowledgement Statistics';

// custom language strings for the plugin
$lang['ackRequired'] = 'You need to acknowledge this page';
$lang['ackText'] = 'I confirm that I have read and understood the contents of this page';
$lang['ackButton'] = 'Acknowledge';
$lang['ackOk'] = 'Page Acknowledged';
$lang['ackGranted'] = 'You acknowledged this page on %s';
$lang['ackDiff'] = 'Show differences since your last acknowledgement from %s';
$lang['ackNotFound'] = 'Nothing found';
$lang['reportTitle'] = 'Acknowledgment Report';
$lang['reportAcknowledgedTitle'] = 'Users who have acknowledged this page';
$lang['reportPendingTitle'] = 'Users who still need to acknowledge this page';
$lang['reportNobody'] = '(none yet)';
$lang['reportUserCount'] = '%d users (show)';
$lang['reportAckedOn'] = 'on';
$lang['overviewPage'] = 'Page';
$lang['overviewUser'] = 'User';
$lang['overviewTime'] = 'Latest acknowledgement';
$lang['overviewMod'] = 'Last Change';
$lang['overviewCurrent'] = 'Current?';
$lang['overviewHistory'] = 'You can access previous acknowledgments by querying the database directly. Ask your administrator for assistance.';
$lang['count'] = 'User %s signed %d of %d pages.';
$lang['yes'] = 'yes';
$lang['home'] = 'Newest Acknowledgements';
$lang['status'] = 'Status';
$lang['all'] = 'all';
$lang['current'] = 'current';
$lang['due'] = 'due';

$lang['toomanyresults'] = 'Too many results to show. Output limited.';

$lang['statsTotal'] = 'Whole wiki';
$lang['statsTotalLink'] = 'Back to top namespace';
$lang['statsNamespace'] = 'Namespace';
$lang['statsSubnamespace'] = 'Sub-namespaces';
$lang['statsRoot'] = '(top level)';
$lang['statsHere'] = '(pages in this namespace)';
$lang['statsAcked'] = 'Acknowledged';
$lang['statsRequired'] = 'Required';
$lang['statsRatio'] = 'Completion';
$lang['statsPages'] = 'Assigned pages';

$lang['pattern'] = 'Page Pattern';
$lang['assignees'] = 'Assignees';
$lang['save'] = 'Save';

$lang['error sqlite plugin missing'] = 'The sqlite plugin is missing';

$lang['notification'] = 'You are required to acknowledge that you have read this page: %s';
