<?php

/**
 * Default settings for the acknowledge plugin
 *
 * @license GPL 2 http://www.gnu.org/licenses/gpl-2.0.html
 * @author  Anna Dabrowska <dokuwiki@cosmocode.de>
 */

$conf['approve_integration'] = 1;
$conf['notification_integration'] = 1;
$conf['onpage_report'] = 'off';
